#include <deque>
#include <sys/time.h>
#include <time.h>
#include <stdint.h>
#include <algorithm>
#include <iostream>
using namespace std;

class DigitalDataRate
{
public:
  explicit DigitalDataRate(int seconds) : d_maxduration(seconds)
  {}

  double getAverage(int seconds, struct timeval* nowp=0)
  {
    fixNowp(nowp);
    cleanup(nowp);
    struct timeval finish(*nowp), start(*nowp);
    start.tv_sec-=seconds;

    struct entry sentry={start, 0};

    entries_t::const_iterator found=lower_bound(d_entries.begin(), d_entries.end(), sentry);

    if(found==d_entries.end() || d_entries.end() - found == 1)
      return 0;

    int32_t previous=found->value;
    struct timeval realStart(found->tv);
    int64_t total=0;

    for(++found; found != d_entries.end(); ++found) {
      total+=found->value - previous; // this works even when wrapped (I hope)
      previous=found->value;
    }
    
    return 1.0*total/ ( (found-1)->tv.tv_sec - realStart.tv_sec + ((found-1)->tv.tv_usec - realStart.tv_usec)/1000000.0);
  }
  void feed(uint64_t bytes, struct timeval* nowp=0)
  {
    feed(static_cast<uint32_t>(bytes), nowp);
  }
  void feed(uint32_t bytes, struct timeval* nowp=0)
  {
    fixNowp(nowp);

    struct entry e={*nowp, bytes};
    d_entries.push_back(e);
  }
  
  void printStatus()
  {
    cerr<<"d_entries size: "<<d_entries.size()<<"\n";
  }

private:
  // non-copyable
  DigitalDataRate(const DigitalDataRate&); 
  DigitalDataRate& operator=(const DigitalDataRate&);

  void fixNowp(struct timeval*& nowp)
  {
    if(!nowp) {
      nowp=&d_tv;
      gettimeofday(nowp,0);
    }
  }

  void cleanup(struct timeval* nowp)
  {
    struct timeval cutoff(*nowp);
    cutoff.tv_sec-=d_maxduration;

    while(!d_entries.empty() && d_entries.front().tv.tv_sec < cutoff.tv_sec)
      d_entries.pop_front();
  }

  struct entry
  {
    struct timeval tv;
    uint32_t value;

    bool operator<(const struct entry& b) const
    {
      return make_pair(tv.tv_sec, tv.tv_usec) < make_pair(b.tv.tv_sec, b.tv.tv_usec);
    }
  };
  typedef deque<struct entry> entries_t;
  entries_t d_entries;
  struct timeval d_tv;
  int d_maxduration;
};


